% G_CLASS - Classification of a feature set.
% 
% Performs a cross-validated SVM classification for a given feature set.
% 
% Copyright (C) 2018 ELTE IK NumAnal, GNU GPL 3

function c = g_class_class(f, use_cv, class_g, class_m)
    n_leads = length(f.train_features);
    
    %% result
    c = struct;
    c.classes = cell(1, n_leads);
    c.accuracy = cell(1, n_leads);
    c.prob_est = cell(1, n_leads);
    c.mat = cell(1, n_leads);
    if use_cv
        c.class_g = zeros(1, n_leads);
        c.class_m = zeros(1, n_leads);
    else
        c.class_g = class_g;
        c.class_m = class_m;
    end
    c.classes_fused = [];
    c.accuracy_fused = 0;
    c.prob_est_fused = [];
    c.mat_fused = [];
    c.accuracy_rejection = 0;
    c.mat_rejection = [];
    c.p_rejection = 0;

    %% settings
    use_lead_fusion = n_leads > 1;
    lead_config = 1:n_leads;
    if use_cv
        cv_fold = 10;
        cv_g = 2.^(3:8);
        cv_m = 2.^(3:8);
        cv_m0 = 16;
    end

    %% classify leads
    for lead = lead_config
        %% cross validate params
        if use_cv
            acc = zeros(size(cv_g));
            for k = 1:length(acc(:))
                acc(k) = g_cv_fold(cv_fold, f.train_classes, f.train_features{lead}, cv_g(k), cv_m0);
            end
            [max_acc, max_k] = max(acc(:));
            max_g = cv_g(max_k);

            acc = zeros(size(cv_m));
            for k = 1:length(acc(:))
                acc(k) = g_cv_fold(cv_fold, f.train_classes, f.train_features{lead}, max_g, cv_m(k));
            end
            [max_acc, max_k] = max(acc(:));
            max_m = cv_m(max_k);
            
            c.class_g(lead) = max_g;
            c.class_m(lead) = max_m;
        end
        %% SVM classification
        [classes1, accuracy1, prob_est1] = g_svm_class(f.train_classes, f.train_features{lead}, f.test_classes, f.test_features{lead}, c.class_g(lead), c.class_m(lead), 1);
        [~, mat1] = g_class_mat(classes1, f.test_classes, []);

        c.classes{lead} = classes1;
        c.accuracy{lead} = accuracy1;
        c.prob_est{lead} = prob_est1;
        c.mat{lead} = mat1;
    end

    %% lead fusion
    if use_lead_fusion
        [c.classes_fused, c.accuracy_fused, c.mat_fused, c.prob_est_fused] = g_class_fusion(c.prob_est{1}, c.prob_est{2}, unique(f.train_classes), f.test_classes, []);
        [c.accuracy_rejection, c.mat_rejection, c.p_rejection] = g_class_rejection(c.classes{1}, c.classes{2}, f.test_classes, c.prob_est_fused, 1, []);
    else
        c.classes_fused = c.classes{1};
        c.accuracy_fused = c.accuracy{1};
        c.mat_fused = c.mat{1};
        c.prob_est_fused = c.prob_est{1};
        
        c.accuracy_rejection = c.accuracy{1};
        c.mat_rejection = c.mat{1};
        c.p_rejection = 0;
    end
end

function [classes, acc, prob_est] = g_svm_class(train_classes, train_features, test_classes, test_features, g, m, b)
    rng(0);
    params = sprintf('-s 0 -t 2 -g %f -c %f -e 0.001 -b %d -m 1000 -q', g, m, b);
    model = libsvmtrain(train_classes(:), train_features, params);
    [classes, acc, prob_est] = libsvmpredict(test_classes(:), test_features, model, sprintf('-b %d -q', b));
    if b
        [~, I] = sort(model.Label);
        prob_est = prob_est(:, I);
    end
    acc = acc(1);
    classes = classes(:)';
end

function [acc] = g_cv_fold(cv_fold, train_classes, train_features, g, m)
    N = size(train_features, 1);
    rng(0);
    P = randperm(N);
    acc1 = zeros(1, cv_fold);
    for j = 1:cv_fold
        range = floor((j-1)*N/cv_fold+1):floor(j*N/cv_fold);
        test_range = P(range);
        train_range = P;
        train_range(range) = [];
        [~, acc1(j), ~] = g_svm_class(train_classes(train_range), train_features(train_range, :), train_classes(test_range), train_features(test_range, :), g, m, 0);
    end
    % acc = mean(acc1);
    acc = min(acc1);
end
